local M = {}

local ct    = nil
local c     = nil
local paint = {}

local bestLapStunt = 0
local bestLapSpeed = 0
local lastLapStunt = 0
local lastLapSpeed = 0
local lastSectorStunt = 0
local lastSectorSpeed = 0
local sectorTimesStunt = {}
local sectorTimesSpeed = {}

--[[ gets current vehicle directory ]]--
local function loadvName()
	dofile( "lua/cmLT/table.save-1.0.lua" )
	local vName, err = table.load( "lua/vName.data" )
	if err == nil then
		return vName["vehicleDirectory"]
		
	else
		print("Error loading map: "..err)
		return {}
	end
end

--[[ gets current vehicle name from name.cs file ]]--
local function loadVehicleName(vDirectory)
	local vName = loadvName()
	if vName ~= nil then
		local file = io.open(vName .. "name.cs", "r") -- open file in read only
		contents = file:read()
		file:close()
		vehicleName = contents
	else
		vehicleName = "N/A"
	end
end
loadVehicleName("vehicleDirectory")

--[[ converts seconds to xx:xx:xx format ]]--
local function timeCount(numSec)
	local nSeconds = numSec
		if nSeconds == 0 then

				coolTime = "00:00:00";
				return coolTime;
		else
			local	nHours = string.format("%02.f", math.floor(nSeconds/3600));
			local	nMins = string.format("%02.f", math.floor(nSeconds/60 - (nHours*60)));
			local	nSecs = string.format("%02.f", math.floor(nSeconds - nHours*3600 - nMins *60));
			local	nHSecs = string.format("%.3f", nSeconds);
			local	hunSecs = string.sub(nHSecs, -3, -2);
				-- coolTime =  nHours..":"..nMins..":"..nSecs
				coolTime =  nMins..":"..nSecs..":"..hunSecs
				return coolTime;
	end
end

--[[ font setup ]]--
local function initPaint()
    local fontName = "lua/cmLT/BebasNeue.otf"
    local tf = SkTypeface.CreateFromFile(fontName, kNormal)
    
	--[[ drop shadows for text visibility ]]--
	paint.fontNormalShadow = SkPaint()
	paint.fontNormalShadow:setColor(0xFFFF0000)
    paint.fontNormalShadow:setAntiAlias(true)
    paint.fontNormalShadow:setARGB(200,0,0,0)
	paint.fontNormalShadow:setTextAlign(kLeft_Align)
    paint.fontNormalShadow:setTextSize(30)
    paint.fontNormalShadow:setTypeface(tf)
	
	paint.fontSmallShadow = SkPaint()
    paint.fontSmallShadow:setAntiAlias(true)
    paint.fontSmallShadow:setARGB(200,0,0,0)
	paint.fontSmallShadow:setTextAlign(kLeft_Align)
    paint.fontSmallShadow:setTextSize(24)
    paint.fontSmallShadow:setTypeface(tf)
	
	--[[ text ]]--
    paint.fontNormal = SkPaint()
	paint.fontNormal:setColor(0xFFFF0000)
    paint.fontNormal:setAntiAlias(true)
    paint.fontNormal:setARGB(240,255,255,255)
	paint.fontNormal:setTextAlign(kLeft_Align)
    paint.fontNormal:setTextSize(30)
    paint.fontNormal:setTypeface(tf)
		
	paint.fontSmall = SkPaint()
    paint.fontSmall:setAntiAlias(true)
    paint.fontSmall:setARGB(240,255,255,255)
	paint.fontSmall:setTextAlign(kLeft_Align)
    paint.fontSmall:setTextSize(24)
    paint.fontSmall:setTypeface(tf)
end

--[[ display while doing speed lap ]]--
local function printTimesSpeed(CP)
    if not ct then
        ct = BeamEngine.canvasTexture
    end
    c = ct:getCanvas()
    ct:clear()
    
    if ( paint.fontNormal == nil ) then
        initPaint()
    end
		
	bestSpeed = timeCount(bestLapSpeed)
	lastSpeed = timeCount(lastLapSpeed)
	lsSpeed = timeCount(lastSectorSpeed)
	bestLapTxtSpeed = string.format("Best: %s", bestSpeed);
	lastLapTxtSpeed = string.format("Last: %s", lastSpeed);
	lastSectorTextSpeed = string.format("Current: %s", lsSpeed);
	
	vNameText = string.sub(vehicleName, 17, -3)	
	vNameTextS = string.format("Driving: %s", vNameText)
	
	--[[ checkpoint stuff ]]--
	cpText = string.format("Checkpoint %s", CP)
	c:drawText(cpText, string.len(cpText), 21, 146, paint.fontSmallShadow)
	c:drawText(cpText, string.len(cpText), 19, 146, paint.fontSmallShadow)
	c:drawText(cpText, string.len(cpText), 20, 146, paint.fontSmall)
		
	--[[ drop shadows ]]--
	c:drawText(vNameTextS, string.len(vNameTextS), 21, 20, paint.fontSmallShadow)
	c:drawText("Corkscrew Mountain", 18, 21, 50, paint.fontNormalShadow)
	c:drawText(bestLapTxtSpeed, string.len(bestLapTxtSpeed), 21, 74, paint.fontNormalShadow)
	c:drawText(lastLapTxtSpeed, string.len(lastLapTxtSpeed), 21, 98, paint.fontNormalShadow)
	c:drawText(lastSectorTextSpeed, string.len(lastSectorTextSpeed), 21, 122, paint.fontSmallShadow)
	
	c:drawText(vNameTextS, string.len(vNameTextS), 19, 20, paint.fontSmallShadow)
	c:drawText("Corkscrew Mountain", 18, 19, 50, paint.fontNormalShadow)
	c:drawText(bestLapTxtSpeed, string.len(bestLapTxtSpeed), 19, 74, paint.fontNormalShadow)
	c:drawText(lastLapTxtSpeed, string.len(lastLapTxtSpeed), 19, 98, paint.fontNormalShadow)
	c:drawText(lastSectorTextSpeed, string.len(lastSectorTextSpeed), 19, 122, paint.fontSmallShadow)
	
	--[[ ]]--
	c:drawText(vNameTextS, string.len(vNameTextS), 20, 20, paint.fontSmall)
	c:drawText("Corkscrew Mountain", 18, 20, 50, paint.fontNormal)
    c:drawText(bestLapTxtSpeed, string.len(bestLapTxtSpeed), 20, 74, paint.fontNormal)
	c:drawText(lastLapTxtSpeed, string.len(lastLapTxtSpeed), 20, 98, paint.fontNormal)
	c:drawText(lastSectorTextSpeed, string.len(lastSectorTextSpeed), 20, 122, paint.fontSmall)
end

--[[ displays best times for current vehicle ]]--
local function printTimesBest(started)
    if not ct then
        ct = BeamEngine.canvasTexture
    end
    c = ct:getCanvas()
    ct:clear()
    
    if ( paint.fontNormal == nil ) then
        initPaint()
    end
	
	
	--[[ converts time to xx:xx:xx ]]--
	bestSpeed = timeCount(bestLapSpeed)
	lastSpeed = timeCount(lastLapSpeed)

	--[[ formats into a string ]]--
	bestLapTxtSpeed = string.format("Best: %s", bestSpeed);
	lastLapTxtSpeed = string.format("Last: %s", lastSpeed);
	
	
	vNameText = string.sub(vehicleName, 17, -3)
	vNameTextS = string.format("Times for: %s", vNameText)
	
	--[[ lap started ]]--
	if started == "Track Started" then
		startedText = string.format("%s", started)
		c:drawText(startedText, string.len(startedText), 171, 122, paint.fontNormalShadow)
		c:drawText(startedText, string.len(startedText), 169, 122, paint.fontNormalShadow)
		c:drawText(startedText, string.len(startedText), 170, 122, paint.fontNormal)
	end	
	
	--[[ drop shadows ]]--
	c:drawText(vNameTextS, string.len(vNameTextS), 21, 20, paint.fontSmallShadow)
	c:drawText("Corkscrew Mountain", 18, 21, 50, paint.fontNormalShadow)
	c:drawText(bestLapTxtSpeed, string.len(bestLapTxtSpeed), 21, 74, paint.fontNormalShadow)
	c:drawText(lastLapTxtSpeed, string.len(lastLapTxtSpeed), 21, 98, paint.fontNormalShadow)
	
	c:drawText(vNameTextS, string.len(vNameTextS), 19, 20, paint.fontSmallShadow)
	c:drawText("Corkscrew Mountain", 18, 19, 50, paint.fontNormalShadow)
	c:drawText(bestLapTxtSpeed, string.len(bestLapTxtSpeed), 19, 74, paint.fontNormalShadow)
	c:drawText(lastLapTxtSpeed, string.len(lastLapTxtSpeed), 19, 98, paint.fontNormalShadow)

	
	--[[ ]]--
	c:drawText(vNameTextS, string.len(vNameTextS), 20, 20, paint.fontSmall)
	c:drawText("Corkscrew Mountain", 18, 20, 50, paint.fontNormal)
    c:drawText(bestLapTxtSpeed, string.len(bestLapTxtSpeed), 20, 74, paint.fontNormal)
	c:drawText(lastLapTxtSpeed, string.len(lastLapTxtSpeed), 20, 98, paint.fontNormal)
end


local function lapComplete(total, trackName)
	if trackName == "corkscrewMountain" then
	
		for i=1, total-1 do
			if sectorTimesSpeed[i] == nil or sectorTimesSpeed[i+1] == nil then
				return false
			elseif sectorTimesSpeed[i] > sectorTimesSpeed[i+1] then
				return false
			end
		end
	end
		
	return true
end

local function loadSpeedTimes()
	dofile( "lua/cmLT/table.save-1.0.lua" )
	local bestSpeedLaps, err = table.load( "lua/cmLT/cmTrackTimes.data" )
	if err == nil then
		return bestSpeedLaps
	else
		print("Error loading map: "..err)
		return {}
	end
end

local function loadTimeBoth()
	local bestSpeedLaps = loadSpeedTimes()
	loadVehicleName("vehicleDirectory")
	vNameText = string.sub(vehicleName, 17, -3)	
	
	--[[ gets best speed lap for current vehicle ]]--
	for _,v in pairs(bestSpeedLaps) do
		if _ == vNameText then
			if bestSpeedLaps[_][1] == "corkscrewMountain" then
				if bestSpeedLaps[_][1] ~= nil then
					bestLapSpeed = bestSpeedLaps[_][2]
				else
					bestLapSpeed = 0
				end
			end
		break
		else
			bestLapSpeed = 0
		end
	end
end

--[[ save vehicle name, track name, and lap time to file ]]--
local function saveTime(levelname, time)
	dofile( "lua/cmLT/table.save-1.0.lua" )
	
	if levelname == "corkscrewMountain" then
		bestSpeedLaps = loadSpeedTimes()
		bestSpeedLaps[vNameText] = { [1] = levelname, [2] = time }
		table.save( bestSpeedLaps, "lua/cmLT/cmTrackTimes.data" )
	end
end

--[[ update routine triggered by checkpoints ]]--
local function update(sector, total, levelname)
    local currentTime = os.clock()
	if levelname == "corkscrewMountain" then
		if lapComplete(total, "corkscrewMountain") then
			lastLapSpeed = currentTime-sectorTimesSpeed[1]
			lastSectorSpeed = 0
			if bestLapSpeed == 0 or lastLapSpeed < bestLapSpeed then
				bestLapSpeed = lastLapSpeed
				
				if levelname ~= nil then
					saveTime(levelname, bestLapSpeed)
				end
			end
		end
		
		sectorTimesSpeed[sector] = currentTime
		if sectorTimesSpeed[1] ~= nil then
			lastSectorSpeed = sectorTimesSpeed[sector]-sectorTimesSpeed[1]
		end
	end
end

M.update    = update 
M.printTimesStunt  = printTimesStunt 
M.printTimesSpeed  = printTimesSpeed
M.loadVehicleName = loadVehicleName
M.loadTimeBoth = loadTimeBoth
M.printTimesBest  = printTimesBest

return M